:- multifile query_description/3.
:- multifile smell_/2.
:- multifile relation_smell_/3.

 
query_description(forbidden_access, relation_smell,'Dependency of given package violates layer access rule.').

          
/** *****************************************************************
  * Metric:  Dependency Inversion Principle
  * Return:  ?PackageID
  * Context: ?PackageID
  *
  * Returns PackageIds of Packages whose dependencies violate the dip.
  */
relation_smell_(forbidden_access, FromId, ToId) :-
    access_rule_violated__(FromPackageId, ToPackageId),
    ri_compilationUnitT_parent(FromPackageId, FromCompilationUnit),
    ri_classT_parent(FromCompilationUnit, FromId),
    ri_compilationUnitT_parent(ToPackageId, ToCompilationUnit),
    ri_classT_parent(ToCompilationUnit, ToId),
    classDependency(FromId, ToId, _).
	
% access_rule_violated__/2
% Returns two PackageIDs that violate the defined accessrules. The PackageDependency Smell guarantees that we get two different IDs.
% ?DependentId 
% ?DependeeId
% 
access_rule_violated__(DependentId, DependeeId) :-
    packageDependency(DependentId, DependeeId, _, _),
    cached_package_layer_assignment(DependentId, FromLayer),
    cached_package_layer_assignment(DependeeId, ToLayer),
    not(FromLayer == ToLayer),
    not(layer_may_access(FromLayer, ToLayer)).



% this detector outputs the dependency statement that violates access rules.
query_description(violating_statements, smell,'Dependency of given package violates layer access rule.').
          
smell_(violating_statements,ViolatingId) :-
    relation_smell(forbidden_access, FromClassId, ToClassId),
    internal_class(FromClassId),
    internal_class(ToClassId),
	(     checkDependencyBetweenClasses(FromClassId, ToClassId, ViolatingId) 
	    ; ( ri_method_parent(ToClassId,MethodId),containsMethDependency__(MethodId,ToClassId,ViolatingId))
	),
	not(getFieldT(ViolatingId,_,_,_,_,_)).


%Alternative #9: method call
% containsMethDependency__/2  returns the callT ID on the given ReferencedClass Method
% +MethodId
% +ReferencedClass
% ?__id
containsMethDependency__(MethodId,ReferencedClass,_id) :-
    ri_callT_encl(MethodId,_id),
	callT(_id, _, _, AnotherCall, _, [], _),
	callT(AnotherCall, _, _, _, _, [], MethodOfAnotherCall),
	methodDef(MethodOfAnotherCall, _, _, _, type(class, ReferencedClass, _), _, _).