/** around_advice(+Predicate, +Before, +After) is det
 * 
 *  changes the semantic of Predicate by adding Before before the original body of Predicate and afterwards calls After .
 *  asserts the new body of Predicate
 *  Implements an AOP around advice
 */
around_advice(Predicate, Before, After) :-
    ground(Before), ground(After), %ground!!
    
	clause(Predicate, Body, Ref), 
	erase(Ref), 
	assert((Predicate :- Before, Body, After)).


/** before_advice(+Predicate, +Before) is det
 * 
 *  changes the semantic of Predicate by adding Before before the original body of Predicate.
 *  Asserts the new body of Predicate.
 *  Implements a AOP before advice
 */
before_advice(Predicate, Before) :-
    ground(Before),
    
	clause(Predicate, Body, Ref), 
	erase(Ref), 
	assert((Predicate :- Before, Body)).
	
/** after_advice(+Predicate, +After) is det
 *
 *  changes the semantic of Predicate by adding After after the original body of Predicate.
 *  Asserts the new body of Predicate.
 *  Implements an AOP after advice
 */		
after_advice(Predicate, After) :-
    ground(After),
    
	clause(Predicate, Body, Ref), 
	erase(Ref), 
	assert((Predicate :- Body, After)).
	
