:- module( cultivate_basic_caching_internal, [ 
													internal_type/1, 
													internal_package/1,
													fill_cache_of_internal_objects/0
									 		 ]).

:- dynamic cached_internal_type/1.
:- dynamic cached_internal_package/1.

/**  fill_cache_of_internal_objects/0 is det
*
* generate caching for all internal objects (package, types,..) for faster lookup.
* @Deprecated internal_package(Package) or internal_type(Type) should be used instead
*             caching is done after the first query.  
*/
fill_cache_of_internal_objects:-
    	fill_internal_packages__,
    	fill_internal_types__.	

/** fill_internal_packages__/0 is det
*
* assert cache for all internal packages.
*  if all the types in the package are internal the package itself is internal.
*  But the empty packages are not internal by definition.
* @Deprecated internal_package(Package) or internal_type(Type) should be used instead
*             caching is done after the first query.  
*/
fill_internal_packages__:-
    	packageT(PackageID,_),
   	    once(type_in_package__(PackageID, Type)),
    	forall(type_in_package__(PackageID, Type), not(externT(Type))),
    		assert( cached_internal_package(PackageID)),
     	fail.
fill_internal_packages__.    	
    					
/** fill_internal_types__/0 is det
*
* assert cache for all internal named types
* @Deprecated internal_package(Package) or internal_type(Type) should be used instead
*             caching is done after the first query.   
*/	
fill_internal_types__:-
		classT(TypeID,_,_,_),
    	not(externT(TypeID)),
    	 	 assert( cached_internal_type(TypeID)),
    	fail.
fill_internal_types__.   

/**
* internal_type(?TypeID)
* succeeds if TypeID is an ID of an internal cached type
*/
internal_type(TypeID):-
		not(cached_internal_type(_)), % no cached types available
		fill_internal_types__,        % fill all cached types 
		!,                           
		cached_internal_type(TypeID). % lookup a cached internal type
		
internal_type(TypeID):-
    	cached_internal_type(TypeID).

/**
* internal_package(?PackageID)
* succeeds if PackageID is an ID of an internal cached package
*/
internal_package(PackageID):-
		not(cached_internal_package(_)), % no cached packages available 
		fill_internal_packages__,        % fill all cached packages
		!, 
		cached_internal_package(PackageID). % lookup a cached internal package
		
internal_package(PackageID):-
    	cached_internal_package(PackageID).
/**
* type_in_package__(+Package,-Type) 
*  local predicate, the same as package_contains_type/2  
*  only here for no "external" dependency in this basic caching all other depend on
*/
type_in_package__(Package,Type):-						 
   ri_compilationUnitT_parent(Package, CompilationUnit),
   ri_classT_parent(CompilationUnit, Type).	


/**
* JHotdraw Measures:  creation of cache vs findall lookup

Cache:
?-time(cultivate_basic_caching_internal:fill_internal_types__).
% 3,364 inferences, 0,00 CPU in 0,00 seconds (0% CPU, Infinite Lips)

Non-Cache:
?-time(findall(_,named_internal_type(_),_)).
% 4,184 inferences, 0,00 CPU in 0,00 seconds (0% CPU, Infinite Lips)

------------------------------------------------------------------------

Cache:
?-time(cultivate_basic_caching_internal:fill_internal_packages__).
% 969 inferences, 0.00 CPU in 0.00 seconds (0% CPU, Infinite Lips)


Non-Cache:
?-time(findall(_,internal_package(_),_)).
% 980 inferences, 0.00 CPU in 0.00 seconds (0% CPU, Infinite Lips)


-----------------------------------------------------------------------

Cache:
?-time(internal_package(A)).
% 1 inferences, 0.00 CPU in 0.00 seconds (0% CPU, Infinite Lips)
A = 17357.

Non-Cache:
?-time(internal_package(A)).
% 259 inferences, 0.00 CPU in 0.00 seconds (0% CPU, Infinite Lips)
A = 17357.


Cache:
?- time(internal_type(A)).
% 1 inferences, 0.00 CPU in 0.00 seconds (0% CPU, Infinite Lips)
A = 17358.

Non-Cache:
?-time(named_internal_type(A)).
% 882 inferences, 0.00 CPU in 0.00 seconds (0% CPU, Infinite Lips)
A = 17358.


*
*/   
   							 