:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(lcom5, metric, 'lack of cohesion of methods').
metric_attributes(lcom5, float(range(0,1), defaultthresholds(0,1))).

% 'Lack of cohesion of methods. Ranges from 0.0 to 2.0. 
% Value of 0.0 => Perfect cohesion.
% Value 1.0 => Complete lack of Cohesion.
% Value 2.0 => None of the attributes are referenced by any method.
% In this case we recommend to split the class, 
% as it probably has more than one functionality.',

metric_(lcom5, ClassId, Value) :-
    normal_class(ClassId),
    lcom5_(ClassId,Value).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% lcom evaluation predicates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

/**
 *  lcom5_(+ClassId, -Value)
 *  
 *  lack of cohesion of methods of a class
 *  
 *  It will bind value to the lcom5 metric 
 *  for the class with ClassId.
 */

% lcom5(class) = (noM(class) - ( (1/nof(class)) * sumNoMaf(class) ) / ( noM(class) - 1 )
lcom5_(ClassId,Value):-
    for_all_fields__sum_number_of_methods_and_constructors_accessing_field(ClassId, NOMAF),
    number_of_methods_and_constructors(ClassId,NOM),
    number_of_fields_without_constants(ClassId,NOF),
    ((NOM = 1; NOF = 0)  -> (Value is 1);
                            (Value is (NOM - ( (1/NOF) * NOMAF ))/(NOM -1))).