:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(newCoh, metric, 'alternative for lcom5 modified by briand').
metric_attributes(newCoh, float(range(0,1), defaultthresholds(0,0.4))).

% 'Lack of cohesion of methods. Ranges from 0.0 to 1.0. Value of 0.0 => Perfect 
% cohesion and Value 1.0 => Complete Lack of Cohesion. In this case we recommend 
% to split the class, as it probably has more than one functionality.',

metric_(newCoh,ClassId, Value):-
    normal_class(ClassId),
	newCoh(ClassId,Value).

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% lcom evaluation predicates
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

/**
 *  newCoh(+ClassId, -Value)
 *  
 *  lack of cohesion of methods of a class
 *  
 *  It will bind value to the newCoh metric 
 *  for the class with ClassId.
 */
% case 1 : nom = 0 or nof = 0 --> no result (to prevent division by zero)
newCoh(ClassId,_) :-
    number_of_methods_and_constructors(ClassId,NOM),
    number_of_fields_without_constants(ClassId,NOF),
    (NOM=0;NOF=0),
    !,
    fail.
    
% case 2 :  newCoh(class) = 1 - sumNoMaf(class)/noM(class)*nof(class)
newCoh(ClassId,Value):-
    for_all_fields__sum_number_of_methods_and_constructors_accessing_field(ClassId, NOMAF),
    number_of_methods_and_constructors(ClassId,NOM),
    number_of_fields_without_constants(ClassId,NOF),
    Value is (NOMAF/(NOM*NOF)).   