:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(numberOfAbstractClasses, metric, 'number of abstract classes').
metric_attributes(numberOfAbstractClasses, int(range(0,infinity), defaultthresholds(0,infinity))).



/**
 * Metric:  numberOfAbstractClasses
 * Return:  ?Integer
 * Context: ?PackageId, ?PackageName
 *
 * Returns the number of abstract classes in the package
 * PackageId/PackageName.
 * For external packages the predicate fails.
 */
metric_(numberOfAbstractClasses, PackageID, NumberOfAbstractClasses) :-
    internal_package(PackageID),
    number_of_abstract_types_in_package(PackageID,NumberOfAbstractClasses).


/**
* number_of_children(+Type,?NOC)
* succeeds if Arg2 is the number of children of type with ID Arg1
**/      		
    


query_description(noc, metric, 'number of children').
metric_attributes(noc, int(range(0,infinity), defaultthresholds(0,infinity))).

/**
 * Metric:  numberOfChildren
 * Return:  ?Integer
 * Context: ?Type
 *
 * Returns the number of children of internal Type
 */
metric_(noc, ClassId, ReturnValue) :-
    internal_type(ClassId),
    is_named(ClassId),
    number_of_children(ClassId,ReturnValue).    
    

query_description(nod, metric, 'number of descendants').
metric_attributes(nod, int(range(0,infinity), defaultthresholds(0,infinity))).

/**
 * Metric:  numberOfDescendants
 * Return:  ?Integer
 * Context: ?Type
 *
 * Returns the number of descendants of internal Type
 */
metric_(nod, ClassId, NOD) :-
    internal_type(ClassId),
    is_named(ClassId),
 	number_of_descendants(ClassId,NOD).


query_description(numberOfInterfaces, metric, 'number of interfaces').
metric_attributes(numberOfInterfaces, int(range(0,infinity), defaultthresholds(0,infinity))).

/** *****************************************************************
  * Metric:  numberOfInterfaces
  * Return:  ?Integer
  * Context: ?PackageId, ?PackageName
  *
  * Returns the number of interfaces in the package
  * PackageId/PackageName.
  * For external packages the predicate fails.
  */
metric_(numberOfInterfaces, PackageID, NumberOfInterfaces) :-
    internal_package(PackageID),
    number_of_interfaces_in_package(PackageID,NumberOfInterfaces).
             	
             	

query_description(number_of_methods, metric, 'number of methods in a class').
metric_attributes(number_of_methods, int(range(0, infinity), defaultthresholds(0, 7))).

/**
 * Metric:  numberOfMethods
 * Return:  ?Integer
 * Context: ?Type
 *
 * Returns the number of methods  of internal Type
 */
metric_(number_of_methods, ClassId, NOM):-
    internal_type(ClassId),
    number_of_methods(ClassId, NOM).
    


query_description(norm, metric, 'number of overridden methods').
metric_attributes(norm, int(range(0,infinity), defaultthresholds(0,infinity))).


/**
 * Metric:  numberOfMethods
 * Return:  ?Integer
 * Context: ?Type
 * Returns the number of overridden methods, a method of a superclass that are overriden in Type.
 */
metric_(norm, ClassID, NORM) :-
    internal_type(ClassID),
    is_named(ClassID),
    number_of_overriden_methods(ClassID,NORM).           	