/** id_of_named_element(+Name,?ID) is det
 *
 *  convenience wrapper for id_of_named_element/3
 */
id_of_named_element(Name,ID):-
      id_of_named_element(Name,_,ID).
      
/** id_of_named_element(+FQNName,?Type,-ID) / 3
 *  
 * Succeeds if ID  can be resolved for specific FQNName and Type  
 *  Package,Class,Method, Field, Param, Parameter of a foreachT, Local
 */
%   Syntax for ARG1: 'a'
id_of_named_element(PackageName,package,ID) :-
    	packageT(ID,PackageName), !.



id_of_named_element(ClassName,class,ID) :-
        globalIds(ClassName,ID),
        classT(ID,_,_,_),
        type_is_in_package(ID,defaultPackage),!.
        

%   Syntax for ARG1:  'a.A'
id_of_named_element(PackageClassName,class,ID) :-
		packageT(PackageID,PackageName),
		package_contains_type(PackageID,ID),
		classT(ID,_,ClassName,_),
		concat_atom([PackageName,ClassName],.,PackageClassName), !.

%   Syntax for ARG1: 'a.A.m'
id_of_named_element(ClassMethodName,method,ID) :-
		packageT(PackageID,PackageName),
		package_contains_type(PackageID,ClassID),
		classT(ClassID,_,ClassName,_),
		methodT(ID,ClassID,MethodName,_,_,_,_),
		concat_atom([PackageName,ClassName,MethodName],.,ClassMethodName), !.
%   Syntax for ARG1:  'a.A.m(p,q)'
id_of_named_element(ClassParamName,method,MethodID) :-
        split_param_method__(ClassParamName,ClassMethodName,ParamName),
        id_of_named_element(ClassMethodName,method,MethodID),
	 	parameter_names(MethodID, ParameterNames),
	 	atomic_list_concat(ParameterNames,',',ParamName),!.
	 	
%   Syntax for ARG1: 'a.A.<init>'
id_of_named_element(ClassMethodName,constructor,ID) :-
		packageT(PackageID,PackageName),
		package_contains_type(PackageID,ClassID),
		classT(ClassID,_,ClassName,_),
		constructorT(ID,ClassID,_,_,_),
		concat_atom([PackageName,ClassName,'<init>'],.,ClassMethodName), !.
						
%   Syntax for ARG1:  'a.A.f'
id_of_named_element(ClassFieldName,field,ID) :-
		packageT(PackageID,PackageName),
		package_contains_type(PackageID,ClassID),
		classT(ClassID,_,ClassName,_),
		fieldT(ID,ClassID,_,FieldName,_),
		concat_atom([PackageName,ClassName,FieldName],.,ClassFieldName), !.
		
%   Syntax for ARG1:  'a.A.m(p)'
id_of_named_element(ClassParamName,param,ID) :-
        split_param_method__(ClassParamName,ClassMethodName,ParamName),
        id_of_named_element(ClassMethodName,method,MethodID),
		methodT(MethodID,_,_,Parameters,_,_,_),
		member(ID,Parameters),
		paramT(ID,MethodID,_,ParamName), !.
    	
%   Syntax for ARG1:  'a.A.m(p)'
id_of_named_element(ClassParamName,param,ID) :-
    	split_param_method__(ClassParamName,ClassMethodName,ParamName),
        id_of_named_element(ClassMethodName,method,MethodID),
		foreachT(ForEachID, _, MethodID, ID, _, _),
		paramT(ID,ForEachID,_,ParamName),!.


    	
    	
		
%   Syntax for ARG1:  'a.A.m/l'		
id_of_named_element(ClassLocalName,local,ID) :-
		localT(ID,_,MethodID,_,LocalName,_),
		id_of_named_element(ClassMethodName,method,MethodID),
		concat_atom([ClassMethodName,LocalName],/,ClassLocalName), !.
		
%  (+Input,-Method,-Param)		
split_param_method__(Input,Method,Param):-
             atom_concat(WithoutRightBracer, ')',Input),
     		 atom_concat(WithLeftBracer, Param,WithoutRightBracer),
     		 atom_concat(Method, '(',WithLeftBracer),     
     		 atomic_list_concat([ Method, '(', Param,')'],Input).						    	