:- multifile query_description/3.
:- multifile smell_/2.

 
query_description(sideeffectsInGettersDetector,smell,'direct or indirect sideeffect in getter method').
  
smell_(sideeffectsInGettersDetector,MethodID) :-
    % find methods -> TODO extract
    packageT(PackageID,_),
    ri_compilationUnitT_parent(PackageID, CompilationUnit),
    ri_classT_parent(CompilationUnit, ClassID),
    not(externT(ClassID)), % prune search
    ri_methodT_parent(ClassID, MethodID),
    % core code
    getter_method(MethodID),
 	first_bad_write_access__(MethodID, _WriteAccess). 
 	 
/**
	method_edits_fields(+MethodID)
    
	This predicate will be true and only true if 
	the method edits one or more fields
*/
method_edits_fields(MethodID, FieldWriteAccess) :-
    methodDef(MethodID, ClassID,_,_,_,_,_),
    not(externT(ClassID)),
    is_side_affecting_operator(FieldWriteAccess, MethodID, EditedField),
    getFieldT(EditedField,_,_,_,_,_),
    !.


is_side_affecting_operator(FieldWriteAccess, MethodID, EditedField):-
    ri_assignT_encl(MethodID,FieldWriteAccess),
    assignT(FieldWriteAccess,_,MethodID,EditedField,_).
is_side_affecting_operator(FieldWriteAccess,MethodID,EditedField):-
    ri_assignopT_encl(MethodID,FieldWriteAccess),
    assignopT(FieldWriteAccess,_,MethodID,EditedField,_,_).
is_side_affecting_operator(FieldWriteAccess,MethodID,EditedField):-
    ri_operationT_encl(MethodID,FieldWriteAccess),
    operationT(FieldWriteAccess,_,MethodID,[EditedField],OpName,_),
    not(OpName = '!').

  
/**
	first_bad_write_access(+MethodID,?WriteAccess)
	
	This will bind WriteAccess to the ID of the Field,
	wich is edited calling the method
*/
first_bad_write_access__(MethodID,WriteAccess) :-
	bagof(_,
		method_has_side_effects__([],MethodID,WriteAccess),
		_).

method_has_side_effects__(VisitedMethods,MethodID,_) :-
	member(MethodID,VisitedMethods),
	!,
	fail.
    
method_has_side_effects__(_VisitedMethods,MethodID,FieldWriteAccess) :-
	method_edits_fields(MethodID,FieldWriteAccess),
	!.

method_has_side_effects__(VisitedMethods,MethodID,FieldWriteAccess) :-
    %actualize the list
    append(VisitedMethods,[MethodID],NewVisitedList),
    %look at all called methods
    ri_callT_encl(MethodID,Id),
    callT(Id,_,MethodID,_,_,_,CalledMethodID),
    %check all called methods
    (method_is_extern(CalledMethodID);
    method_has_side_effects__(NewVisitedList,CalledMethodID,FieldWriteAccess)).
    

    
/**
 * native methods can have side effects, it cannot be checked so we assume
 * that the current one has a side effect.
 */
%method_has_side_effects(_VisitedMethods,MethodID,FieldWriteAccess) :-
    %look at all called methods
%    ri_callT_encl(MethodId,_Id),
%    callT(_Id,_,MethodID,_,_,_,CalledMethodID),
%	modifierT(CalledMethodID,'native),
%	!.
	    