/**
 * Author: Andreas Gasper
 */

/**
 * is_controlflow_affecting(+StatementID)
 *
 * Succeeds if the Statement with StatementID is a controlflow affecting statement
 */
    
is_controlflow_affecting(StatementID):-
    ast_node_type_value(StatementID, 'ifT').

is_controlflow_affecting(StatementID):-
    ast_node_type_value(StatementID, 'forT').

is_controlflow_affecting(StatementID):-
    ast_node_type_value(StatementID, 'foreachT').

is_controlflow_affecting(StatementID):-
    ast_node_type_value(StatementID, 'whileT').

is_controlflow_affecting(StatementID):-
    ast_node_type_value(StatementID, 'doWhileT').

is_controlflow_affecting(StatementID):-
    ast_node_type_value(StatementID, 'switchT').
    
/**
 * Kommentare von dsp:
 *
 * Ist soweit ok und ordentlich dokumentiert.
 *
 * Einige Vorschlge:
 * - Wenn du die einzelnen Flle so aufzhlst, knntest du auch die PEFs nehmen. (Eingestandener Vorteil deiner
 *   Implementierung: Du bist unabhngig von der Stelligkeit der PEFs.)
 * - Wenn man die PEFs verwenden wrde, wre das Prdikat etwas effizienter, wenn man die Variable ungebunden 
 *   lassen wrde.
 * - Mir ist die Verwendung von ast_node_type_value an dieser Stelle unsympathisch, weil
 *   sie grundlos auf einen hheren Abstraktionsgrad geht als ntig. Solltest du sie dennoch benutzen wollen,
 *   dann nutze sie auch: 
 *   ast_node_type_value(Statement, NodeType), member(NodeType, ['ifT', 'forT', 'forEachT', 'whileT', 'doWhileT', 'switchT']).
 * - Das Prdikat gehrt natrlich ins bisher nicht existierende classification/statements :-)
 * - Da wir berall mit IDs arbeiten, knnen wir das "ID" auch konsequent weglassen.
 * - Zum Thema "control flow affecting" findest du Kommentare in method_contains_control_statement.pl
 */