/**
 * Author: Andreas Gasper
 */

/**
 * method_contains_control_statement(+MethodID, -StatementID)
 *
 * Succeeds if the Statement is a statements that can affect the controlflow 
 * and is within the method with MethodID
 */

method_contains_control_statement(MethodID, StatementID):-
    methodDef(MethodID, _, _, _, _, _, _),
    ri_ifT_encl(MethodID, StatementID).

method_contains_control_statement(MethodID, StatementID):-
    methodDef(MethodID, _, _, _, _, _, _),
    ri_forT_encl(MethodID, StatementID).

method_contains_control_statement(MethodID, StatementID):-
    methodDef(MethodID, _, _, _, _, _, _),
    ri_foreachT_encl(MethodID, StatementID).

method_contains_control_statement(MethodID, StatementID):-
    methodDef(MethodID, _, _, _, _, _, _),
    ri_whileT_encl(MethodID, StatementID).

method_contains_control_statement(MethodID, StatementID):-
    methodDef(MethodID, _, _, _, _, _, _),
    ri_doWhileT_encl(MethodID, StatementID).

method_contains_control_statement(MethodID, StatementID):-
    methodDef(MethodID, _, _, _, _, _, _),
    ri_switchT_encl(MethodID, StatementID).
 
 /**
  * Kommentare von dsp:
  *
  * Gefllt mir gut. Klar. Effizient. Dokumentiert.
  * 
  * Ein paar Vorschlge fr Weiterentwicklungen: 
  *   - Das Prdikate wrde gut in containment passen. (Allerdings sind da bis auf Sebastians Ergnzungen
  *     nur Interface-Element (und lokale Klassen) drin. Das wre ntzlich von Statements getrennt zu halten,
  *     um die bersicht nicht zu gefhrden. Idee?) 
  *  AG: Mir ist nicht klar was du unter containment verstehst. 
  *
  *   - Konsistenterer Name mit containment/in: method_contains_control_statement(Method, Statement)
  *     Das wre auch geradeliniger als "controlflow_affecting". 
  *  AG: Einverstanden.
  *
  *   - Wir sollten uns darauf einigen, an welchen Stellen wir den Typ von Variablen prfen.
  *     Mein Vorschlag wre, das hier berall in containment zu tun. Dann wre also berall eine erste Zeile 
  *     "methodDef(Method, _, _, _, _, _, _)," zu ergnzen.
  *  AG: Einverstanden.
  *
  *   - Da wir berall mit IDs arbeiten, knnen wir das "ID" auch konsequent weglassen.
  *  AG: Ich kann mit der ID irgendwie besser denken...
  *
  *   - Denk- & Nachschlagarbeit: Knnen wir alle statements dieser Art klassifzieren? "controlflow affecting" 
  *     wren ja auch mindestens breakT, continueT, caseT... 
  *  	AG: breakT, continueT, caseT... fhren dazu das der Kontrollfluss in einem "Ast" abgebrochen bzw. wieder zusammmengefhrt wird.
  *      	Die controlflow affecting Statements sind Statements an denen sich der Kontrolfluss verzweigt.
  *
  *		Ein Ad-hoc Vorschlag zu den oben von dir aufgezhlten
  *     Statements, den du aber gerne z.B. anhand der Java Language Specification prfen solltest:
  *       branch statements = {switch, if-else}
  *       loop statements   = {while, do-while, for, foreach}
  *     Wenn du hier keine geeignete Quelle finden solltest, dann lass uns noch mal gemeinsam suchen. 
  *  AG: Ich sehe hier im Moment keine Notwendigkeit die Statements weiter zu klassifizieren, lasse mich aber gerne vom Gegenteil berzeugen.
  */

