:- module('cultivate_concepts_term_collocations',[ collocation_measure/3, dominating_collocation/3]).
:- use_module(term_model).

% + - - 
dominating_collocation(Term,ATerm,Value):-
    compute_cooccurrence_measure(Term,Measure),
    max_term(Measure,(ATerm,Value)).
% + -    
compute_collocation_measure(Term,Measure):-
    setof((ATerm,Result), (cooccurrence_measure(Term,ATerm,Result),not(Result==0)),Measure).

% + + ?    
max__((Term,Value),(_,AValue),(Term,Value)):-
    Value > AValue,!.
max__((_,_),(ATerm,AValue),(ATerm,AValue)).

% + ?
max_term(List,Max):- max_term(List,('',0),Max).    

% Accumulator version used from the above max_term/2
max_term([],Max,Max).    
max_term([Head|Tail],CurrentMax,ListMax) :-
    max__(Head,CurrentMax,NewMax),
    max_term(Tail,NewMax,ListMax).

%% cooccurrence_measure(+Term,?AnotherTerm,?Percentage) is det
%
% calculates the cooccurrence measure between two different terms
%
% the result is    #join(Term1,Term2)/(#Term1+#Term2-join(Term1,Term2)
%        1  if both co-occur whenever either term occurs.
%        0  if both terms do not co-occur (terms are mutually exclusive).
%        >0 if both co-occur (terms are non mutually exclusive).
collocation_measure(Term,AnotherTerm,Percentage) :-
    element_has_term(Element,Term),
    element_has_term(Element,AnotherTerm),
    not(Term == AnotherTerm),
    count(term_join(Term,AnotherTerm),Join),
    count( (ri_element_has_term(Term,_)), TermCount),
    count( (ri_element_has_term(AnotherTerm,_)), AnotherTermCount),
    Percentage is Join / (TermCount + AnotherTermCount - Join).    

term_join(Term,AnotherTerm):-
    ground([Term,AnotherTerm]),
    not(Term == AnotherTerm),
    ri_element_has_term(Term,Element),
    ri_element_has_term(AnotherTerm,Element).

term_join(Term,AnotherTerm):-
    ground(Term),
    var(AnotherTerm),
    ri_element_has_term(Term,Element),
    element_has_term(Element,AnotherTerm),
    not(Term == AnotherTerm).

term_join(Term,AnotherTerm):-
    var(Term),
    var(AnotherTerm),
    element_has_term(Element,Term),
    element_has_term(Element,AnotherTerm),
    not(Term == AnotherTerm).

