:- module( cultivate_concepts_terms_model, [ 
				fill_term_model/0,
				classify_term/2,
				interface_term/1,
				behavior_term/1,
				external_term/1
				
              ] ).
%:-consult(terms/utils).
:-consult(utils/ig_term_occurrences).

:- dynamic interface_term/1. % (termid)
:- dynamic behavior_term/1. % (termid)
:- dynamic external_term/1. % (termid)


classify_term(Id,interface):-
    interface_term(Id).
classify_term(Id,behavior):-
    behavior_term(Id).
classify_term(Id,external):-
    external_term(Id).
    

clear_term_model :-
	retractall( interface_term(_)),
	retractall( behavior_term(_)),
	retractall( external_term(_)).

report_term_cache_contents_statistics :-
   forall( 
     dynamic_local_predicate(Head),
     ( functor(Head,Functor,Arity),
       count(Head,Count),
       format('~a  term cache entries for ~w.~n', [Count,Functor/Arity])
     ) 
   ).



fill_term_model :- 
	clear_term_model,
	compute_additional_model_information__,
	%fill_interface_terms__,
	%fill_behavior_terms__,
	generate_ig_term_occurrence_counters__ ,
	report_term_cache_contents_statistics.
	

classify_term__(Id,TermId):-	
		once((element_context__(Id,Context),
    		 ast_node_type_value(Context,Type),
    		 is_extern(Type,Context))
    		 ),
    	assert(external_term(TermId)),
    	!.
classify_term__(Id,TermId):-	
		is_interface_level(Id),
    	assert( interface_term(TermId)),
    	!.
classify_term__(_,TermId):-	
		assert( behavior_term(TermId)),
    	!.     
    	
    	   	    	
compute_additional_model_information__ :-
    cultivate_concepts_term_cache:element_has_lexical(Id,TermId,_),
    	classify_term__(Id,TermId),
    fail.	
compute_additional_model_information__.    	
/**
additional_introduce_in_class :-
	 relevant_context_element(Context),
	 classT(Context,_,_,_),
	 cultivate_concepts_term_counts:term_occurrences_in_context(Context,interface,Term,Count),
	 Count > 0,
	 not(cultivate_concepts_term_cache:context_has_introduced_term(Context, Term)),
	 (
	 forall( ( cultivate_concepts_term_counts:term_occurrences_in_context(Context,interface,OtherTerm,OtherCount),OtherCount > 0, not(Term == OtherTerm)),
	 		 ( cultivate_concepts_term_cache:context_has_introduced_term(OtherContext, OtherTerm)))
	 )
	 -> assert( cultivate_concepts_term_cache:context_has_introduced_term(Context, Term) ) ;
	 	true, 		 
	 	format(' ~w ~n',[Term]),
	 fail. 
additional_introduce_in_class.
*/


 
	 
              