:- module( 'cultivate_concepts_terms', [
            cache_available/0,
            no_cache_available/0,
            assert_term/1,
            assert_term_unique/1,
            term/1,
            clear_term_cache/0,
       
            is_compound/1,
            termid/3,
            retractall_term/1,
            
            context_has_term/3,
            
            relevant_context_element/1, 
 	        context_has_lexical/3,
            element_has_lexical/3,
            element_has_term/2,
            lexical_term/2,
            lexical_compound/2,
		    ri_element_has_lexical/2,   
          
            ri_element_has_term/2

			  ] ).

% Dynamic definitions     
%==================================================================
:- dynamic relevant_context_element/1.
:- dynamic context_has_lexical/3. % (Context,TermId)
:- dynamic context_has_term/3. %(Context,Term,ProgramElementId) 
:- dynamic element_has_lexical/3.  % (ProgramElementId,TermId,PositionInContextName)
:- dynamic lexical_term/2.            %(TermId,Term)
%:- dynamic context_has_lexical/2.     % (ContextId,TermId)
:- dynamic lexical_compound/2.    % (ProgramElementId,[ListOfTerms])
:- dynamic element_has_term/2.  % (ElementId,Term) (without duplicates) (17403 with Name 'getGetGetMethod' yields (17403,get), (17403,method)

% For Part Of Speech (only for internal terms)
:- dynamic lexical_has_lemma/2.  % (TermId,Lemma) e.g. (stm17403p1,take)
:- dynamic lexical_has_pos/2.    % (TermId,PartOfSpeech)  (stm17403p1,VV)


% Reverse indices for optimization
:- dynamic ri_element_has_lexical/2.     % (TermId,ProgramElementId) with duplicates
%:- dynamic ri_context_has_term/2. %(Term,Context)
:- dynamic ri_element_has_term/2.  % (Term, ElementId) (without duplicates)

:- use_module('../util/termcache_utils').
:- use_module('../util/identifier').  	 
%TODO
% Singular/Plural howto store?
% POS Tagging
%==================================================================

clear_term_cache:-
    retractall(relevant_context_element(_)),
    retractall(context_has_lexical(_,_,_)),
    retractall(lexical_term(_,_)),
    retractall(lexical_compound(_,_)),
    retractall(ri_element_has_lexical(_,_)),
    retractall(ri_lexical_term(_,_)),
    
    retractall(context_has_term(_,_,_)),
    retractall(element_has_lexical(_,_,_)).

/**
* no_cache_available/0  - succeeds iff no cached term for id fact can be found
* cache_available/0     - succeeds iff any cached term for id fact can be found
*/
no_cache_available :-
      not(lexical_term(_,_)).
      
cache_available :-
      once(lexical_term(_,_)).   
      
      
/** assert_term(+Fact) is det 
 *
 *  asserts the Fact in the terms module
 */      
assert_term(Fact):-
    assert(Fact).

/** assert_term_unique(+Fact) is det 
 *
 *  asserts the Fact in the terms module, no assert if the Fact already exists
 */  
assert_term_unique(Fact):-
    assert_unique(Fact). 

    
/** term(+Fact) is det 
 *
 *  calls the Fact in the terms module
 *  simple delegation to the terms module
 */      
term(Fact):-
   call(Fact).      

retractall_term(Fact):-
    retractall(Fact).

/** is_not_compound(+Name) is det
 *  
 * succeeds if Name is not a compound e.g model or my
 */
is_compound(Name):-
    identifier_built_of_terms(Name,List),
    length(List,1),!.

    
%  + + ?      
termid(Element,Position,TermId):-
 	atomic_list_concat(['stm',Element,'p',Position],TermId).    
 	
