package org.cs3.cultivate.dev.logging;

import org.cs3.cultivate.model.prolog.*;
import org.cs3.cultivate.model.prolog.query.*;
import org.cs3.cultivate.nature.implementation.IState;
import org.cs3.cultivate.core.repository.*;
import org.eclipse.core.runtime.IStatus;

public aspect CultivateLogging extends Logging {

	CultivateLogging(){
		super("[Cultivate]");
	}
	
	pointcut scope(): within(org.cs3.cultivate..* && !org.cs3.cultivate.dev..*);

	pointcut svensCode(): within(org.cs3.cultivate.reftoadap..*);

	pointcut dontJudgeCode(): svensCode();

	pointcut interestingExecution(): 
		// execution(* CultivateNature.initIfRequired()) 
		// || execution(* StateMachine.*(..)) || 
		execution(public * *..FactBaseUpdatedListener.*(..)) 
		|| execution(public static * *..FactBaseUpdatedListener.*(..)) 
		|| execution(* JTransformerProjectListener.*(..));

	pointcut interestingCall(): 
		// call(* *..CountDownLatch.*(..)) || 
		call(* JTUtils.getPrologInterface(..));

	pointcut logStatus(IStatus status): execution(public static void *..Logger.log(IStatus)) && args(status);

	before(IStatus status): logStatus(status) {
		log("logging status = " + status);
	}

	pointcut automaticStateTransition(IState onState):
		call(* IState.execute(..)) && target(onState);

	IState around(IState onState): automaticStateTransition(onState)  {
		logBefore("preparing for automatic state transition on state = "
				+ simpleClassName(onState));
		IState result = proceed(onState);
		logAfter("result of automatic state transition is "
				+ simpleClassName(result));
		return result;
	}

	pointcut exceptionSwallowingInStateMachine(Exception e): 
		withincode(* IState.execute(..)) 
		&& (handler(Exception) && args(e));

	before(Exception e): exceptionSwallowingInStateMachine(e){
		log("About to swallow " + simpleClassName(e) + ".");
		e.printStackTrace();
	}

	pointcut svensConditionalTransformations(): execution(public Term org.cs3.cultivate.model.prolog.transformation..*Transformation.getTerm());

	after() returning(Term conditionalTransformation): svensConditionalTransformations()  {
		log("Conditional Transformation = '" + conditionalTransformation + "'.");
	}

   after(String taskDescription) returning: scope() && call(* IProgressMonitor.subTask(..)) && args(taskDescription) {
	   log("Next subtask = " + taskDescription);
   }
	
   
   after(IPrologQuery query, QueryResult queryResult) returning : 
	   call(void IPrologQuery.apply(..)) 
	   && target(query) && args(queryResult)
	   && cflow(execution(* Repository.consultCache(..))){
	   
	   System.out.println("Resolved query:" + query + " from cache: " + queryResult.getPrologResult());
   }
      
   after(IPrologQuery query) returning(QueryResult queryResult):
	   call(* *.get(Object))
	   && args(query)
	   && cflow(execution(void Repository.executeAndNotify(..))) {
	   
	   System.out.println("Resolved query:" + query + " from cache: " + queryResult.getPrologResult());
   }	
}
