package org.cs3.cultivate.dev.logging;

import java.util.List;
import java.util.Map;

import org.cs3.cultivate.model.prolog.IPrologQuery;
import org.cs3.cultivate.model.prolog.pdtimplementation.PDTQueryExecutor;
import org.cs3.pl.prolog.PrologSession;



public aspect PrologLogging extends Logging pertypewithin(PDTQueryExecutor){

	pointcut nowhere(): execution(* absolutly.Nowhere.nothing());

	pointcut scope(): nowhere();

	pointcut interestingExecution(): nowhere();

	pointcut interestingCall(): nowhere();

	pointcut dontJudgeCode(): nowhere();

	PrologLogging() {
		super("[CV-Prolog]");
	}

	before(String query): call(* PrologSession.queryOnce(..)) && args(query){
		logBefore("queryOnce(" + query + ")");
	}

	after() returning (Map result): call(* PrologSession.queryOnce(..)) {
		logAfter("result =" + result);
	}

	before(String query): call(* PrologSession.queryAll(..)) && args(query){
		logBefore("queryAll(" + query + ")");
	}

	after() returning (List result): call(* PrologSession.queryAll(..)) {
		logAfter("result =" + result);
	}
	
	after() throwing (Exception e): call(* PrologSession.query*(..)){
		logAfter("failed. Exception = " + e.getLocalizedMessage());
	}
	
	
	after() throwing (Exception e): call(* PrologSession.query*(..)) {
		logAfter("failed. Exception = " + e.getLocalizedMessage());
		throw new RuntimeException(e);
	}

	before(IPrologQuery query): execution(* execQuery*(..)) && args(PrologSession, query, String) {
		logBefore("Execution a query of type " + query.getClass().getSimpleName());
	}
	
	after(): execution(* execQuery*(..)) {
		logBefore("Execution of the query done.");
	}
}
