:- multifile query_description/3.
:- multifile smellc_/3.
:- multifile metric_attributes/2.

/* query_description(deprecated_dependends_only, smell, '').
   */     

/**
 * TODO, dsp, 19.10.09: Define a smell that does find all member
 * that are depended by at least one deprecated and then by only 
 * by deprecated elements.
 * 
 * The goal was to develop a tool to successively find deletable
 * code. @Deprecated seemed to be a good first choice as an 
 * an annotation. On a second look this choice doesn't seem to be
 * so good, as deprecated elements are still meant to live for a 
 * while.
 * 
 * A first choice would be just to look for those elements that
 * are directly used by deprecated code. Later on we could as
 * well look for code that is indirectly used by deprecated code.
 * In this case we need to calculate dependency SCCs first, or
 * otherwise e.g. two methods that call each other recursively
 * but are otherwise only called by deprecated code, would not
 * be found.
 * 
 * Instead of static dependencies we might as well use a conservative
 * approach to dynamic dependencies like in the refactoring
 * for adaptivity add-on. (Idea: A method declaration or implementation
 * is "using" all implementing or overriding methods in subclasses
 * to fulfill its job.)  
 * 
 * A definitly meaningful smell would be not deleted unused
 * deprecated code.
 * 
 * Ununsed code is worth another smell. :-)
 */

/**
 *  the_annotation_type_deprecated(-AnnotationTypeDeprecated)
 *
 *  Succeeds if AnnotationTypeDeprecatedis the (ID of) the type declaring the "@Deprecated" annotation.
 *                                   
 *  This predicate simply wraps the look up of 'java.lang.Deprecated', to avoid spelling
 *  errors, when referencing this type declaration. 
 */ 

the_annotation_type_deprecated(AnnotationTypeDeprecated) :-
    fullQualifiedName(AnnotationTypeDeprecated, 'java.lang.Deprecated').
    
/**
 *  element_or_containing_type_has_annotation(+Element, ?AnnotationType)
 *
 *  Succeeds if the element Element is an annotated field, method or type that 
 *  has an annotation of type AnnotationType. It succeeds as well if a type 
 *  in which this element is has this annotation.
 *
 *  For example a method should be considered deprecated if she is 
 *  deprecated or the whole containing class is deprecated.  
 */

/**
 * TODO, dsp, 19.10.09: The following predicates seem to work for 
 * the standard situation one thinks of.
 *  
 * A thorough review of all elements that can be annotated would 
 * be useful. Alternatively we could express the reduced scope by 
 * naming this predicate member_...
 *
 * Annotations are by default NOT inherited. This seems to make sense
 * here. Does it? We are not checking super types! 
 * 
 */

element_or_containing_type_has_annotation(Element, AnnotationType) :-
    annotatedT(Element, Annotation),
    !,
    annotationT(Annotation, _, _, AnnotationType, _).

element_or_containing_type_has_annotation(Element, AnnotationType) :-
    (
      type_is_in_type_transitive(Element, ContainingType);
      field_is_in_type(Element, ContainingType); 
      method_is_in_type(Element, ContainingType)
    ),
    element_or_containing_type_has_annotation(ContainingType, AnnotationType).

/**
 * TODO, dsp, 19.10.09: The following predicates are just used for tests till now. 
 * They should be moved somewhere to basic/prolog/containment
 */

    
/**
 *  type_contains_member(+Type, ?Member)
 *
 *  Succeeds if the type Type contains the field, method, or type Member.
 *
 */ 
    
type_contains_member(Type, Member) :-
    (type_contains_type(Type, Member); type_contains_method(Type, Member); type_contains_field(Type, Member)).
 
 /**
  * type_contains_member_transitive(+Type, ?Member)
  * 
  * Succeeds if the type Type or a type somewhere in this type contains the field, method, or type Member.
  *
  */   
type_contains_member_transitive(Type, Member) :-
    type_contains_member(Type, Member).
    
type_contains_member_transitive(Type, Member) :-
    type_contains_type_transitive(Type, ContainedType),
    type_contains_member(ContainedType, Member).

/*
 * Instead name_of_element could be used. 
 * (TODO, dsp, 19.10.09: name_of_element does not return the "<init>" fr constructors. Do we want that?)
 */
member_name(Member, Name) :-
    (classT(Member, _, Name, _); methodDef(Member, _, Name, _, _, _, _); fieldT(Member, _, _, Name, _)).