:- multifile query_description/3.
:- multifile smellc_/3.
:- multifile metric_attributes/2.

query_description(rcm_dip, relation_smell, 'Dependency Inversion Principle Violation').
query_description(rcm_adp, relation_smell, 'Acyclic Dependencies Principle Violation').
query_description(rcm_sdp, relation_smell, 'Stable Dependencies Principle Violation').
          
/** *****************************************************************
  * Metric:  Dependency Inversion Principle
  * Return:  ?PackageID
  * Context: ?PackageID
  *
  * Returns PackageIds of Packages whose dependencies violate the dip.
  */
relation_smell_(rcm_dip, DependentId, DependeeId) :-
	packageDependency(DependentId, DependeeId,_,_),
	metric(abstractness,DependentId, ReturnValueDependentPackage),
	metric(abstractness, DependeeId, ReturnValueDependeePackage),
	ReturnValueDependentPackage > ReturnValueDependeePackage.

 /** *****************************************************************
  * Metric:  Acyclic Dependency Principle
  * Return:  ?PackageID
  * Context: ?PackageID
  *
  * Returns PackageIds of Packages whose dependencies violate the adp.
  */

relation_smell_(rcm_adp, DependentId, DependeeId) :-
    packageDependency(DependentId, DependeeId,SccNumber,_),
    SccNumber > 0.

 /** *****************************************************************
  * Metric:  Stable Dependency Principle
  * Return:  ?PackageID
  * Context: ?PackageID
  *
  * Returns PackageIds of Packages whose dependencies violate the sdp.
  */
relation_smell_(rcm_sdp, DependentId, DependeeId) :-
       packageDependency(DependentId, DependeeId,_,_),
       metric_(instability,DependentId,InstabilityDependentPackage),
	   metric_(instability,DependeeId, InstabilityDependeePackage),
	   InstabilityDependentPackage < InstabilityDependeePackage.
	