:- multifile detector_/4.
:- multifile detector_description/4.

%Depending class: accessing class; ReferencedClass: accessed class; DependencyDegree: Count of Dependency statements between DependingClass & ReferencedClass
detector_description(classDependency,'one class that depends on another',class,unrated).

%detector_(classDependency,occurrence,[(class,?DependingClass),(referenced_class,?ReferencedClass), (dependencyDegree,?DependencyDegree)],[])
detector_(classDependency,occurrence,[(class,DependingClass),(referenced_class,ReferencedClass), (dependencyDegree,DependencyDegree)],[]) :-
    classDependency(DependingClass, ReferencedClass).
    
/*
* classDependency(+DependingClass, -ReferencedClass, -DependencyDegree)
* MIGRATION - this is a special query, not a smell/metric (detector before). 
* Do we have a special concept for this?
*/
classDependency(DependingClass, ReferencedClass, DependencyDegree):- 
    internal_class(DependingClass),
    internal_class(ReferencedClass),
    not(ReferencedClass = DependingClass),
	checkDependencyBetweenClasses_(DependingClass,ReferencedClass),
	findall(Id, checkDependencyBetweenClasses(DependingClass,ReferencedClass,Id), List), length(List, DependencyDegree).

detector_description(classUsageDependency,'one class that uses parts of another',class,unrated).

detector_(classUsageDependency,occurrence,[(class,DependingClass),(referenced_class,ReferencedClass), (dependencyDegree,DependencyDegree)],[]) :-
    internal_class(DependingClass),
    internal_class(ReferencedClass),
    not(ReferencedClass = DependingClass),
	checkDependencyBetweenClasses_(DependingClass,ReferencedClass),
	not(internal_class_implements_interface__(DependingClass, ReferencedClass)),
	not(internal_interface_extends_interface__(DependingClass, ReferencedClass)),
	not(internal_class_extends_class__(DependingClass, ReferencedClass)),
	
	findall(Id, checkDependencyBetweenClasses(DependingClass,ReferencedClass,Id), List), 
	length(List, DependencyDegree).	
	
internal_class(ClassID) :- classT(ClassID,_,_,_), not(externT(ClassID)), not(anonymous_class(ClassID)).

/**
 * checkDependencyBetweenClasses(+DependingClass,+ReferencedClass)
 *      
 * checks if there is a dependency from 'DependingClass' 
 * to 'ReferencedClass'. Possible dependencies are:
 * 	- field declaration
 *  - parameter of a method
 *  - local variable in a method
 *  - direct access to class in method
 *  - direct access to class in field initializer (TODO)
 *  - direct access to class in static initializer (TODO)
 */
%Alternative #1: Dependent on superclass

checkDependencyBetweenClasses_(DependingClass,ReferencedClass) :-
    once(checkDependencyBetweenClasses(DependingClass,ReferencedClass,_)).
	
%Alternative #1: Dependent on superclass
checkDependencyBetweenClasses(DependingClass,ReferencedClass,_id) :-
    extendsT(DependingClass,ReferencedClass), 
    _id = DependingClass.

checkDependencyBetweenClasses(DependingClass,ReferencedClass,_id) :-
    implementsT(DependingClass,ReferencedClass),
    _id = DependingClass.

%Alternative #2: Dependent on field type
checkDependencyBetweenClasses(DependingClass,ReferencedClass,FieldId) :-
    ri_fieldT_parent(DependingClass,FieldId),
    fieldT(FieldId,DependingClass, type(class,ReferencedClass,_), _,_).

%Alternative #3-11: Dependent on result, argument or local var type of a method
checkDependencyBetweenClasses(DependingClass,ReferencedClass,_id) :-
    ri_methodT_parent(DependingClass,MethodId),
    containsMethDependency(MethodId,ReferencedClass,_id).
  
%Alternative #4: Dependent on access to super/this/class usage of the referenced class
checkDependencyBetweenClasses(DependingClass,ReferencedClass,CallID) :-
	ri_methodT_parent(DependingClass,MethodId),
	ri_blockT_parent(MethodId, BlockID),
	ri_execT_encl(BlockID, ExecID),
	ri_callT_parent(ExecID, CallID),
	callT(CallID,ExecID,_,SelectID,_,_,_),
	selectT(SelectID,CallID,_,class,type(class,ReferencedClass,_),_).

/**
 * containsMethDependency_(+MethodId,+ReferencedClass)
 */

%Alternative #3: Dependent on type of a method parameter
containsMethDependency(MethodId,ReferencedClass,_defId) :-
    ri_paramT_parent(MethodId,_defId),
    paramT(_defId,MethodId,type(class, ReferencedClass,_),_).

%Alternative #4: Dependent on type of a local variable in a method
containsMethDependency(MethodId,ReferencedClass,_defId) :-
   ri_localT_encl(MethodId,_defId),
    localT(_defId,_,MethodId,type(class, ReferencedClass,_),_,_).

%Alternative #5: access to field
containsMethDependency(MethodId,ReferencedClass,_id) :-
    ri_getFieldT_encl(MethodId,_id),
    getFieldT(_id,_,MethodId,_,_,FieldId),
    ri_fieldT_parent(ReferencedClass,FieldId).
    
%Alternative #6: return type
containsMethDependency(MethodId,ReferencedClass,MethodId) :-
    methodT(MethodId,ContainingClass,_,_,type(class, ReferencedClass,_),_,_).
    
%Alternative #7: type cast
containsMethDependency(MethodId,ReferencedClass,_defId) :-
    ri_typeCastT_encl(MethodId,_defId),
    typeCastT(_defId,_,MethodId,type(class, ReferencedClass,_),_).
    
%Alternative #8: type test
containsMethDependency(MethodId,ReferencedClass,_defId) :-
    ri_typeTestT_encl(MethodId,_defId),
    typeTestT(_defId,_,MethodId,type(class, ReferencedClass,_),_).

%Alternative #9: method call
containsMethDependency(MethodId,ReferencedClass,_id) :-
    ri_callT_encl(MethodId,_id),
    callT(_id,_,MethodId,_,_,_,CalledMethodId),
    methodT(CalledMethodId,ReferencedClass,_,_,_,_,_).
  
%%Alternative #9: method call
%containsMethDependency(MethodId,ReferencedClass,_id) :-
%    ri_callT_encl(MethodId,_id),
%	callT(_id, _, _, AnotherCall, _, [], _),
%	callT(AnotherCall, _, _, _, _, [], MethodOfAnotherCall),
%	methodDef(MethodOfAnotherCall, _, _, _, type(class, ReferencedClass, _), _, _).
  
%Alternative #10: constructor call
containsMethDependency(MethodId,ReferencedClass,_id) :-
    ri_newClassT_encl(MethodId,_id),
    newClassT(_id,_,MethodId,CalledMethodId,_,_,_,_),
    constructorT(CalledMethodId,ReferencedClass,_,_,_).
    
%Alternative #11: array creation
containsMethDependency(MethodId,ReferencedClass,_id) :-
    ri_newArrayT_encl(MethodId,_id),
    newArrayT(_id,_,MethodId,_,_,type(class, ReferencedClass,_)).
     
containsMethDependency(MethodId,ReferencedClass,_id) :-
    ri_identT_encl(MethodId,_id),
    identT(_id,_,MethodId,_,type(class, ReferencedClass,_)).